@@ -41,7 +41,7 @@ enum Seg {
     // "::"
     Ellipsis,
     // *1":" 1*4HEXDIG "."
-    MaybeV4,
+    MaybeV4(bool),
     // ":"
     SingleColon,
 }
@@ -340,9 +340,9 @@ impl<'a> Parser<'a> {
                     }
                     ellipsis_i = i;
                 }
-                Some(Seg::MaybeV4) => {
-                    if i > 6 {
-                        // Not enough space.
+                Some(Seg::MaybeV4(colon)) => {
+                    if i > 6 || colon == (i == ellipsis_i) {
+                        // Not enough space, triple colons, or no colon.
                         return None;
                     }
                     let octets = self.scan_v4()?.octets();
@@ -409,7 +409,7 @@ impl<'a> Parser<'a> {
                         i += 1;
                         continue;
                     }
-                    _ if b == b'.' => return Some(Seg::MaybeV4),
+                    _ if b == b'.' => return Some(Seg::MaybeV4(colon)),
                     _ => break,
                 }
             } else {
@@ -703,5 +703,9 @@ mod tests {
         assert!(parse_v6("1:2:3:4:5:127.0.0.1").is_none());
         // too many groups
         assert!(parse_v6("1:2:3:4:5:6:7:127.0.0.1").is_none());
+        // triple colons before v4
+        assert!(parse_v6(":::4.4.4.4").is_none());
+        // no colon before v4
+        assert!(parse_v6("::ffff4.4.4.4").is_none());
     }
 }
